@@ -153,6 +153,7 @@ use core::{
     convert::{Infallible, TryFrom},
     fmt,
     marker::{PhantomData, PhantomPinned},
+    mem::ManuallyDrop,
     num::{
         NonZeroI128, NonZeroI16, NonZeroI32, NonZeroI64, NonZeroI8, NonZeroU128, NonZeroU16,
         NonZeroU32, NonZeroU64, NonZeroU8,
@@ -283,6 +284,16 @@ impl<C: ?Sized> CheckBytes<C> for PhantomPinned {
     }
 }
 
+impl<C: ?Sized, T: CheckBytes<C> + ?Sized> CheckBytes<C> for ManuallyDrop<T> {
+    type Error = T::Error;
+
+    #[inline]
+    unsafe fn check_bytes<'a>(value: *const Self, c: &mut C) -> Result<&'a Self, Self::Error> {
+        let _ = T::check_bytes(value as *const T, c)?;
+        Ok(&*value)
+    }
+}
+
 /// An error resulting from an invalid boolean.
 ///
 /// Booleans are one byte and may only have the value 0 or 1.
@@ -417,7 +428,7 @@ macro_rules! impl_tuple {
             #[allow(clippy::unneeded_wildcard_pattern)]
             unsafe fn check_bytes<'a>(value: *const Self, context: &mut C) -> Result<&'a Self, Self::Error> {
                 let field_bytes = ($(ptr::addr_of!((*value).$index),)+);
-                $($type::check_bytes(field_bytes.$index.cast::<$type>(), context).map_err($error::$type)?;)+
+                impl_tuple!(@check_fields field_bytes, $error, context, $($type $index,)*);
                 Ok(&*value)
             }
         }
@@ -426,7 +437,12 @@ macro_rules! impl_tuple {
             [$($error_rest,)*],
             [$($type $index,)+]
         }
-    }
+    };
+    (@check_fields $field_bytes:ident, $error:ident, $context:ident,) => {};
+    (@check_fields $field_bytes:ident, $error:ident, $context:ident, $type:ident $index:tt, $($type_rest:ident $index_rest:tt,)*) => {
+        impl_tuple!(@check_fields $field_bytes, $error, $context, $($type_rest $index_rest,)*);
+        $type::check_bytes($field_bytes.$index, $context).map_err($error::$type)?;
+    };
 }
 
 impl_tuple! {
@@ -490,7 +506,7 @@ impl<T: fmt::Display> fmt::Display for SliceCheckError<T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             SliceCheckError::CheckBytes { index, error } => {
-                write!(f, "check failed for slice index {}: {}", index, error)
+                write!(f, "check failed for slice index {index}: {error}")
             }
         }
     }
@@ -534,7 +550,7 @@ impl fmt::Display for StrCheckError {
     #[inline]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            StrCheckError::Utf8Error(e) => write!(f, "utf8 error: {}", e),
+            StrCheckError::Utf8Error(e) => write!(f, "utf8 error: {e}"),
         }
     }
 }
@@ -582,7 +598,7 @@ impl fmt::Display for CStrCheckError {
     #[inline]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            CStrCheckError::Utf8Error(e) => write!(f, "utf8 error: {}", e),
+            CStrCheckError::Utf8Error(e) => write!(f, "utf8 error: {e}"),
             CStrCheckError::MissingNullTerminator => write!(f, "missing null terminator"),
         }
     }
@@ -698,18 +714,16 @@ impl<T: fmt::Display> fmt::Display for EnumCheckError<T> {
                 inner,
             } => write!(
                 f,
-                "check failed for enum struct variant {}: {}",
-                variant_name, inner
+                "check failed for enum struct variant {variant_name}: {inner}"
             ),
             EnumCheckError::InvalidTuple {
                 variant_name,
                 inner,
             } => write!(
                 f,
-                "check failed for enum tuple variant {}: {}",
-                variant_name, inner
+                "check failed for enum tuple variant {variant_name}: {inner}"
             ),
-            EnumCheckError::InvalidTag(tag) => write!(f, "invalid tag for enum: {}", tag),
+            EnumCheckError::InvalidTag(tag) => write!(f, "invalid tag for enum: {tag}"),
         }
     }
 }
